function [C,R,N] = EMD(tt,xx,Nimf,iimax)
%[C,R,N]=function EMD(tt,xx,jmax,imax)
% EMD signal decomposition
%
% search for local maxima in time series
locMax = @(x) ...
    1+find(x(2:end-1)>x(3:end) & x(2:end-1)>=x(1:end-2));
%
% pre-allocate storage for residual R and IMFs C
R=zeros(length(xx),Nimf);
C=0*R;
% we use N to remember number of iterstions required oper IMF
N=zeros(1,Nimf);
%
% we sart with original time series
R(:,1)=xx;
for jj=1:Nimf-1 % upper limit must be one less than the number of IMFs
    %
    x1=R(:,jj); % get residual
    for ii=1:iimax % try max 20 times to find IMF
        xo=x1;              % data set to be worked on
        lmx=locMax(xo);     % find local maxima
        lmn=locMax(-xo);    % find local minima
        % check if we already got IMF
        if length(find(xo(lmn)>0))+...
           length(find(xo(lmx)<0))==0, break,end
        % construct envelope by interpolating extrema
        xmx=interp1(tt(lmx),xo(lmx),tt,'cubic','extrap');
        xmn=interp1(tt(lmn),xo(lmn),tt,'cubic','extrap');
        % Constract new data set by centring inbetween envelope  
        xm=(xmx+xmn)/2;
        x1=xo-xm;
    end
    %Store number of iterations, IMS and residual
    N(jj)=ii;
    C(:,jj)=x1;
    R(:,jj+1)=R(:,jj)-x1;
    %
    % if fistr attempt yieded to IMF this should be the last one
    if ii==1, break,end
end
%
