%Scr4_11
%ifl=0; %1-8,0
root='../pam_book_data/';

fname='S_coe_short'; tname='S_coeruleoalba';

[xx,fs]=wavread([root fname]);
xx=xx(:,1);
tt=(0:length(xx)-1)'/fs;

[B,F,T]=specgram(xx-mean(xx),1024,fs,hann(256),192);

%get spectral power
B2=B.*conj(B);

if ifl==1 %Figure 4.18
figure(1)
subplot(211)
plot(tt,xx-mean(xx),'k');
ylabel('Pressure [rel]')
xlim(tt([1 end]))

subplot(212)
imagesc(T,F/1000,10*log10(B2)), axis xy
cl=caxis;
colormap(1-gray)
caxis(cl(2)+[-50 0])
xlabel('Time [s]')
ylabel('Frequency [kHz]')
return
end

if ifl==3 %Fig 4.20
ifr=find(F>=12570,1,'first'); xl=[0 1];
itm=find(T>=0.595,1,'first'); 
else
ifr=find(F>=10850,1,'first'); xl=[2 3];
itm=find(T>=2.5,1,'first');
end

if ifl==2 || ifl==3 %Fig 4.19-20
    figure(1)
    subplot(211)
    plot(T,10*log10(B2(ifr,:)),'k'), if ~isempty(xl),xlim(xl);end
    grid on
    xlabel('Time [s]')
    ylabel('Level [dB]')
    subplot(212)
    plot(F/1000,10*log10(B2(:,itm)),'k')
    xlim(F([1 end])/1000)
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Level [dB]')
    return
end
%
%spectral noise estimation
f1=1000; f2=4000;
if1=find(F>=f1,1,'first');
if2=find(F>=f2,1,'first');
if 0
    n1=mean(10*log10(B2(if1,:)));
    n2=mean(10*log10(B2(if2,:)));
else
    n1=10*log10(mean(B2(if1,:)));
    n2=10*log10(mean(B2(if2,:)));
end

q1=10.^((n2-n1)/20);
Fo=(q1*f2-f1)/(1-q1);
NEo=n1+20*log10(f1+Fo);

NE=10.^((NEo-20*log10((F+Fo)))/10);

if ifl==4 %Figure 4.21
    figure(1)
    plot(F/1000,10*log10(B2(:,itm)),'k')
    line(F/1000,10*log10(NE),'color','k','linewidth',2)
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Level [dB]')
    title(sprintf('%.1f - 20 log10(F+%.1f)',NEo,Fo))
    return
end
%
%normalize
B3=B2./(NE*ones(size(T')));
if ifl==5 %Figure 4.22
    figure(1)
    imagesc(T,F/1000,10*log10(B3)), axis xy
    cl=caxis;
    colormap(1-gray)
    caxis(cl(2)+[-50 0])
    xlabel('Time [s]')
    ylabel('Frequency [kHz]')
    return
end

%2nd derivative
df=fs/1024;
D2=2*B3(2:end-1,:)-(B3(3:end,:)+B3(1:end-2,:));
D2=[D2(1,:);D2;D2(end,:)];

if 0
figure(1)
    subplot(211)
    plot(T,(D2(ifr,:)),'k'), if ~isempty(xl),xlim(xl);end
    grid on
    xlabel('Time [s]')
    subplot(212)
    plot(F/1000,(D2(:,itm)),'k')
    grid on
    xlabel('Frequency [kHz]')
return
end

itm1=find(T>=2.54,1,'first');
itm2=find(T>=0.595,1,'first'); 

if ifl==6 %Figure 4.23
figure(1)
clf
subplot(211)
plot(F/1000,D2(:,itm1),'k')
subplot(212)
plot(F/1000,B3(:,itm2)*max(D2(:,itm2))/max(B3(:,itm2)),'k:',F/1000,D2(:,itm2),'k')
xlim(F([1 end])/1000)
xlabel('Frequency [kHz]')
return
end

%thresholding the 2nd derivative
th=4;
B4=(D2>th).*B2;

%find local maxima in frequency direction
M1=[0*B4(1,:);B4(2:end,:)>B4(1:end-1,:)];
M2=[B4(1:end-1,:)>B4(2:end,:);0*B4(end,:)];
M=M1.*M2; clear M1 M2 % clear intermediate matrices


if ifl==7 %Figure 4.24
figure(1)
clf
imagesc(T,F/1000,B4>0),axis xy
colormap(1-gray)
cl=caxis;
caxis([0 cl(2)]);
xlabel('Time [s]')
ylabel('Frequency [kHz]')
return
end

if ifl==8%Figure 4.25
figure(1)
clf
imagesc(T,F/1000,M),axis xy
colormap(1-gray)
cl=caxis;
caxis([0 cl(2)]);
xlabel('Time [s]')
ylabel('Frequency [kHz]')
    xlim([0 1.1])
    ylim([5 15])
return
end

% tracking
% select some time samples from dataset
iox=150:200;%1:1600; %1600 equiv 1.1 s on spectrogram
th=11; %threshold
mx=6; %max idle length
gain=0.5; % filter gain
beta=0.2; % exponential averaging
tic
[DET,ntr]=doTracking(M,iox,th,mx,gain,beta);
toc

% clean up short tracks
for ii=1:ntr
    ifl=DET(:,3)==ii;
%    [ii sum(ifl)]
    if sum(ifl)<10 %6+3
        DET(ifl,:)=[];
    end
end

if 1%ifl==0
    [Mr,Mc]=find(M);
    %
    figure(1)
    hold off
    hp=plot(T(Mc),F(Mr)/1000,'k.');% set(hp(1:2),'color',0.5*[1 1 1]);
    hold on
    cc=['ko-';'ks-';'kv-';'k^-'];
    kk=0;
    for ii=min(DET(:,3)):max(DET(:,3))
        idt=DET(:,3)==ii;
        if sum(idt)>0
            kk=kk+1;
            plot(T(DET(idt,1)),F(round(DET(idt,2)))/1000,cc(1+mod(kk-1,4),:),'markersize',4)
        end
    end
    xlim(T(iox([1 end])))
    ylim(F([1 513])/1000)
    xlabel('Time [s]')
    ylabel('Frequency kHz]')
    hold off
return
end
