% load data
[xx,fs] = wavread('../Pam_book_data/zifioSelectScan.wav');
%
% construct time vector
tt = (1:length(xx))'/fs;
%
% subtruct mean to remove constant offset
ss = xx - mean(xx);
%
%calibrate time series
cal=10^(27/20); %unit is equiv 147 dB re 1 uPa
ss=cal*ss;
%
%noise estimate over complete time series
nn_o=sqrt(mean(ss.^2));
%
% estimate signal-to-noise ratio
snr_o = abs(ss)/nn_o;
%
% envelope
yy_a=doAnalytic(ss,512);
nn_a=sqrt(mean(ss.^2));
snr_a=abs(yy_a)/nn_a;
%
yy=ss;
uu=cumsum(yy.^2);

ds=2.5e-5;
iw=2*ceil(fs*ds/2);
zz1=sqrt((uu(1+iw:end)-uu(1:end-iw))/iw);
ttz1=tt(1+iw/2:end-iw/2);

nn_e1=sqrt(mean(zz1.^2));
snr_e1=zz1./nn_e1;

ds=2e-4;
iw=2*ceil(fs*ds/2);
zz2=sqrt((uu(1+iw:end)-uu(1:end-iw))/iw);
ttz2=tt(1+iw/2:end-iw/2);

nn_e2=sqrt(mean(zz2.^2));
snr_e2=zz2./nn_e2;

%KaiserTaeger
yy=0*ss;
for ii=2:length(xx)-1
    yy(ii)=max(0,ss(ii)*ss(ii)-ss(ii-1)*ss(ii+1));
end

nn_e3=sqrt(mean(yy));
snr_e3=sqrt(yy)./nn_e3;

%
th=4.5;
dt_max=2e-3;
%
tsig=[0.1076 0.1633 0.6361 0.6915 1.0731 1.1288 1.5106 1.5662 ...
      1.9422 1.9976 2.4173 2.4733 2.8850 2.9409];

if ifl==1 %set to 1 to activate processing
dets0=[];
dets1=[];

thv=2:0.5:30;
for th=thv
    th
    [jth1,i13,i23]=do_SimpleDetection1(snr_o,tt,th,dt_max);
    [jth2,i13,i23]=do_SimpleDetection1(snr_a,tt,th,dt_max);
    [jth3,i13,i23]=do_SimpleDetection1(snr_e1,ttz1,th,dt_max);
    [jth4,i13,i23]=do_SimpleDetection1(snr_e2,ttz2,th,dt_max);
    [jth5,i13,i23]=do_SimpleDetection1(snr_e3,tt,th,dt_max);
    [jth6,i13,i23]=do_PageDetection(yy_a,6,4*th^2,1e-3);

    cnt=zeros(1,6);
    for ii=1:length(tsig)
        if find(abs(tsig(ii)-tt(jth1))<0.001), cnt(1)=cnt(1)+1; end
        if find(abs(tsig(ii)-tt(jth2))<0.001), cnt(2)=cnt(2)+1; end
        if find(abs(tsig(ii)-ttz1(jth3))<0.001), cnt(3)=cnt(3)+1; end
        if find(abs(tsig(ii)-ttz2(jth4))<0.001), cnt(4)=cnt(4)+1; end
        if find(abs(tsig(ii)-tt(jth5))<0.001), cnt(5)=cnt(5)+1; end
        if find(abs(tsig(ii)-tt(jth6))<0.001), cnt(6)=cnt(6)+1; end
    end

    dets0=[dets0;[length(jth1),length(jth2),length(jth3),length(jth4),length(jth5),length(jth6)]];
    dets1=[dets1;cnt];
end
end

pdet=dets1/14;
pfa=(dets0-dets1)./dets0;

figure(1)
subplot(211)
hp=plot(thv,pfa(:,1:5),'k.-','markersize',4);
set(hp(1),'Marker','*')
set(hp(2),'Marker','o')
set(hp(3),'Marker','d')
set(hp(4),'Marker','s')
set(hp(5),'Marker','v')
ylabel('P_{FA}')
legend('Abs','Env','RMS_{25}','RMS_{200}','TK');
subplot(212)
hp=plot(thv,pdet(:,1:5),'k.-','markersize',4);
set(hp(1),'Marker','*')
set(hp(2),'Marker','o')
set(hp(3),'Marker','d')
set(hp(4),'Marker','s')
set(hp(5),'Marker','v')

xlabel('threshold')
ylabel('P_{det}')
 
figure(2)
plot(pfa,pdet,'.-')
hp=plot(pfa,pdet,'k.-','markersize',6);
set(hp(1),'Marker','*')
set(hp(2),'Marker','o')
set(hp(3),'Marker','d')
set(hp(4),'Marker','s')
set(hp(5),'Marker','v')
set(hp(6),'Marker','>')
xlabel('P_{FA}')
ylabel('P_{det}')
legend('Abs','Env','RMS_{25}','RMS_{200}','TK','Page');
