%Scr4_9
%ifl=0; % 1 to 5; 0 is ROC
% load data
[xx,fs] = wavread('../Pam_book_data/zifioSelectScan.wav');
%
% construct time vector
tt = (1:length(xx))'/fs;
%
% subtruct mean to remove constant offset
ss = xx - mean(xx);
clear xx
%
%calibrate time series
cal=10^(147/20); %unit is equiv 147 dB re 1 uPa
ss=cal*ss;
%
% estimate power spectrum
its=floor([0.25 0.6]*fs);
[Bo,F]=psd(ss(its(1):its(2)),1024,fs/1000,hann(256),192);
Po=10*log10(Bo)-10*log10(fs);

xxo=doAnalytic(ss,512);
nno=sqrt(mean(abs(xxo).^2));
snro = abs(xxo)/nno;

if ifl==1
    figure(1)
    plot(F,Po,'k','linewidth',2)
    ylim([15 60])
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Spectral density [dB re 1\muPa^2/Hz]')
    return
end

%equalization
%Butterworth filter of fixed order
[b1,a1]= butter(2, 2*45000/fs, 'high');
[b2,a2]= butter(2, 2*120000/fs, 'low');
yy1=filter(b1,a1,ss);
yy2=filter(b2,a2,yy1);


xx2=doAnalytic(yy2,512);
nn2=sqrt(mean(abs(xx2).^2));
snr2 = abs(xx2)/nn2;

[B1,F]=psd(yy1(its(1):its(2)),1024,fs/1000,hann(256),192);
[B2,F]=psd(yy2(its(1):its(2)),1024,fs/1000,hann(256),192);
%
P1=10*log10(B1)-10*log10(fs);
P2=10*log10(B2)-10*log10(fs);

if ifl==2
    figure(1)
    hp=plot(F,Po,'k',F,P1,'k',F,P2,'k');
    set(hp(1),'linewidth',1,'linestyle','-')
    set(hp(2),'linewidth',2,'linestyle','--')
    set(hp(3),'linewidth',2,'linestyle','-')
    ylim([15 60])
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Spectral density [dB re 1\muPa^2/Hz]')
    legend('Data','45 kHz HP','45 kHz HP + 120kHz LP')
    return
end

%broad band-pass Butterworth filter
df1=[20 60]/fs*2000; 
df2=[10 100]/fs*2000;
xx3=BP_filter(ss,df1,df2);
nn3=sqrt(mean(abs(xx3).^2));
snr3 = abs(xx3)/nn3;
%
[B3,F]=psd(real(xx3(its(1):its(2))),1024,fs/1000,hann(256),192);
%
P3=10*log10(B3)-10*log10(fs);

if ifl==3
    figure(1)
    hp=plot(F,Po,'k',F,P3,'k');
    set(hp(2),'linewidth',2)
    ylim([15 60])
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Spectral density [dB re 1\muPa^2/Hz]')
    return
end

%narrow band-pass Butterworth filter
df1=[37 43]/fs*2000; 
df2=[30 50]/fs*2000;
xx4=BP_filter(ss,df1,df2);
nn4=sqrt(mean(abs(xx4).^2));
snr4 = abs(xx4)/nn4;
%
[B4,F]=psd(real(xx4(its(1):its(2))),1024,fs/1000,hann(256),192);
%
P4=10*log10(B4)-10*log10(fs);
%
if ifl==4
    figure(1)
    hp=plot(F,Po,'k',F,P4,'k');
    set(hp(2),'linewidth',2)
    ylim([15 60])
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Spectral density [dB re 1\muPa^2/Hz]')
    return
end
%
% plot both filters together
if ifl==5
    figure(1)
    hp=plot(F,Po,'k',F,P3,'k',F,P4,'k');
    set(hp(2),'linewidth',2)
    set(hp(3),'linewidth',2,'linestyle','--')
    ylim([15 60])
    grid on
    xlabel('Frequency [kHz]')
    ylabel('Spectral density [dB re 1\muPa^2/Hz]')
    return
end
%

%ROC curves
th=4.5;
dt_max=2e-3;
%
tsig=[0.1076 0.1633 0.6361 0.6915 1.0731 1.1288 1.5106 1.5662 ...
      1.9422 1.9976 2.4173 2.4733 2.8850 2.9409];

if 1 %set to 1 if precessing is needed
dets0=[];
dets1=[];

thv=2:0.5:30;
for th=thv
    th
    [jtho,i1o,i2o]=do_SimpleDetection1(snro,tt,th,dt_max);
    [jth2,i12,i22]=do_SimpleDetection1(snr2,tt,th,dt_max);
    [jth3,i13,i23]=do_SimpleDetection1(snr3,tt,th,dt_max);
    [jth4,i14,i24]=do_SimpleDetection1(snr4,tt,th,dt_max);

    cnt=zeros(1,4);
    for ii=1:length(tsig)
        if find(abs(tsig(ii)-tt(jtho))<0.001), cnt(1)=cnt(1)+1; end
        if find(abs(tsig(ii)-tt(jth2))<0.001), cnt(2)=cnt(2)+1; end
        if find(abs(tsig(ii)-tt(jth3))<0.001), cnt(3)=cnt(3)+1; end
        if find(abs(tsig(ii)-tt(jth4))<0.001), cnt(4)=cnt(4)+1; end
    end

    dets0=[dets0;[length(jtho),length(jth2),length(jth3),length(jth4)]];
    dets1=[dets1;cnt];
end
end

pdet=dets1/14;
pfa=(dets0-dets1)./dets0;
%
figure(1)
set(gcf,'position',[100 100 560 600])
subplot(211)
hp=plot(thv,pfa,'k.-');
set(hp(1),'Marker','*','markersize',4)
set(hp(2),'Marker','o','markersize',4)
set(hp(3),'Marker','s','markersize',4)
set(hp(4),'Marker','d','markersize',4)
ylabel('P_{FA}')
legend('Envelope','Noise Equalization','Broad BP filter','Narrow BP filter')
subplot(212)
hp=plot(thv,pdet,'k.-');
set(hp(1),'Marker','*','markersize',4)
set(hp(2),'Marker','o','markersize',4)
set(hp(3),'Marker','s','markersize',4)
set(hp(4),'Marker','d','markersize',4)

xlabel('threshold')
ylabel('P_{det}')

figure(2)
hp=plot(pfa(:,1:2),pdet(:,1:2),'k.-');
set(hp(1),'Marker','*','markersize',5)
set(hp(2),'Marker','o','markersize',5)
xlabel('P_{FA}')
ylabel('P_{det}')
legend('Envelope','Noise Equalization')
return
