%Scr6_6
% sumulation
tt=0:3600;
vv=1.5+0*tt;
beta=0*tt;
gamma=30+0*tt;

beta(tt<10*60)=-80;
beta(tt>=10*60 & tt<40*60)=0;
beta(tt>=40*60 & tt<50*60)=80;
beta(tt>=50*60)=0;

randn('state',0); %to reproduce random numbers

sigs=1;
dx=vv.*sin(gamma*pi/180).*cos(beta*pi/180)...
    +sigs*randn(size(tt));
dy=vv.*cos(gamma*pi/180).*cos(beta*pi/180)...
    +sigs*randn(size(tt));
dz=vv.*sin(beta*pi/180)...
    +sigs*randn(size(tt));

xx=cumsum(dx);
yy=cumsum(dy);
zz=cumsum(dz); zz(zz>0)=0;

tcl=tt(tt>60 & tt<43*60);

xcl=interp1(tt,xx,tcl);
ycl=interp1(tt,yy,tcl);
zcl=interp1(tt,zz,tcl);

xo=[xcl;ycl;zcl];

h0=[-1000,-1000,-500];
h1=[3000,0,-100];
h2=[0,3000,-100];
h3=[3000,3000,-100];
h4=[1000,1000,-100];

%plot track and hydrophones
figure(1)
hold off
hp=plot3(xx,yy,zz,'k',xcl,ycl,zcl,'k.');
set(hp(1),'color',0.5*[1 1 1])
grid on
set(gca,'CameraPosition',[11000 -13000 6000])
%
hold on
plot3([h0(1) h1(1) h2(1) h3(1) h4(1)],...
      [h0(2) h1(2) h2(2) h3(2) h4(2)],...
      [h0(3) h1(3) h2(3) h3(3) h4(3)],...
      'ko','markersize',10)
hold off
%
xlabel('X')
ylabel('Y')
zlabel('Z')
box on

% simulate measurements
%obtain acoustic ranges
R0=sqrt((xcl-h0(1)).^2+(ycl-h0(2)).^2+(zcl-h0(3)).^2);
R1=sqrt((xcl-h1(1)).^2+(ycl-h1(2)).^2+(zcl-h1(3)).^2);
R2=sqrt((xcl-h2(1)).^2+(ycl-h2(2)).^2+(zcl-h2(3)).^2);
R3=sqrt((xcl-h3(1)).^2+(ycl-h3(2)).^2+(zcl-h3(3)).^2);
R4=sqrt((xcl-h4(1)).^2+(ycl-h4(2)).^2+(zcl-h4(3)).^2);
%
%obtain measured delays
sigm=2*15;
dR1=R1-R0+sigm*randn(size(R0));
dR2=R2-R0+sigm*randn(size(R0));
dR3=R3-R0+sigm*randn(size(R0));
dR4=R4-R0+sigm*randn(size(R0));
%
if 0
figure(2)
plot(tcl,dR1,'-',tcl,dR2,'-',tcl,dR3,'-',tcl,dR4,'-')
xlabel('Time [s]')
ylabel('Delay [m]')
end
%
% initiate Kalman filter
dt=diff(tcl); dt=[dt(1) dt];
% state transfer matrix
A=[1 0 0 0  dt(1) 0     0     0;
   0 1 0 0  0     dt(1) 0     0;
   0 0 1 0  0     0     dt(1) 0;
   0 0 0 1  0     0     0     dt(1);
   0 0 0 0  1     0     0     0;
   0 0 0 0  0     1     0     0;
   0 0 0 0  0     0     1     0;
   0 0 0 0  0     0     0     1];
% measurement vector
C=2*[0 h1-h0 0 0 0 0;
     0 h2-h0 0 0 0 0;
     0 h3-h0 0 0 0 0;
     0 h4-h0 0 0 0 0];
%
% need this for direct inversion
b0=[h1*h1';
    h2*h2';
    h3*h3';
    h4*h4']-h0*h0';

C0=2*[0 h1-h0;
     0 h2-h0;
     0 h3-h0;
     0 h4-h0];
%store here determinant
D=0*tcl;

%Kalman filter
%initialize state vector
xh=[0 0 0 0 0 0 0 0]'*tcl;
xh(:,1) = [1500 6 15 -100 1 1 1 -1]';
xh0=xh(1:4,:);
%
% measurement error covariance matrix
R=(1e+4)^2*eye(4);
%
Q=diag([1 1 1 .01 .1 .1 .1 .01].^2);
%
I=eye(8);
P=1*I;
for ii=2:length(tcl)
    %adjust state model
    A(1,5)=dt(ii);
    A(2,6)=dt(ii);
    A(3,7)=dt(ii);
    A(4,8)=dt(ii);
    %
    %adjust measurement model
    dR=[dR1(ii) dR2(ii) dR3(ii) dR4(ii)]';
    C(:,1)=2*dR;
    y=b0-dR.^2;
    %
    % prediction
    xhs=A*xh(:,ii-1); % state
    ys=C*xh(:,ii-1); %measurement
    Ps=A*P*A'+Q; % covariance matrix
    %
    % correction
    K=Ps*C'*inv(C*Ps*C'+R);
    xh(:,ii)=xhs+K*(y-ys);
    P=(I-K*C)*Ps;
    %
    % direct solution for comparison only
    C0(:,1)=C(:,1);
    D(ii)=det(C0);
    xh0(:,ii)=pinv(C0)*y;
    %
end

if 0
figure(3)
hp=plot(tcl,xh0(1,:),tcl,xh(1,:),'r',tcl,R0,'b');
set(hp(1),'color',0.5*[1 1 1]);
set(hp(2:3),'linewidth',2)
xlabel('Time [s]')
ylabel('Range [m]')
end

figure(4)
hp=plot(tcl,xh0(2:4,:),'k',tcl,xo','k:',tcl,xh(2:4,:),'k');
set(hp(1:3),'color',0.5*[1 1 1]);
set(hp(4:end),'linewidth',2)
text(tcl(end)+100,xh(2,end),'X','fontweight','bold')
text(tcl(end)+100,xh(3,end),'Y','fontweight','bold')
text(tcl(end)+100,xh(4,end),'Z','fontweight','bold')
xlabel('Time [s]')
ylabel('Distance [m]')
