%Scr9_7
%
Rmax=7000;
%
vo=1.5; %speed [m/s]
nsim=3600; %number of clicks / dive
%
%heading distribution
dg=0.01;
gam=-pi:dg:pi;
k=5;
wg=1/(2*pi*besseli(0,k))*exp(k*cos(gam));
cwg=cumsum(wg)*dg;
icwg=cwg>0.0001 & cwg<0.9999;
%
% pitch distribution
bet=-pi/2:dg:pi/2;
k=5;
wb=1/(2*pi*besseli(0,k))*exp(k*cos(bet));
cwb=cumsum(wb)*dg;
icwb=cwb>0.0001 & cwb<0.9999;
%
% number of simulations
nrep=100000;
% % needed for histograms
dx=10;
hx=0:dx:Rmax;
do=1;
ho=(0:do:180)/180*pi;
%
if 0 % set to 1 to carry out simulation
    clear Ho
else
    load('Scr9_7dat')
end
%
if ~exist('Ho','var')
    Ho=zeros(length(hx),length(ho));
    tic
    rand('state',1)
    for ii=1:nrep
        % simulate initial track position
        %  randomly located within circle
        ro=inf;
        while ro>Rmax
            rdo=rand(1,2);
            xo=2*Rmax*(rdo(1,1)-0.5);
            yo=2*Rmax*(rdo(1,2)-0.5);
            zo=500;
            ro=sqrt(xo^2+yo^2+zo^2);
        end
        %simulate initial direction
        gamo=2*pi*rand(1,1);
        %
        %simulate animal track
        ncl=rand(nsim,2);

        %simulate animal track
        arg=ncl(:,1);
        head=gamo+...
            interp1(cwg(icwg),gam(icwg),arg,'lin','extrap');
        %
        arg=ncl(:,2);
        pitch=...
            interp1(cwb(icwb),bet(icwb),arg,'lin','extrap');
        %
        %assume constant speed between clicks ans 2 clicks/s
        dro=vo*0.5;
        dxa=dro*cos(head).*cos(pitch);
        dya=dro*sin(head).*cos(pitch);
        dza=dro*sin(pitch);

        %estimate track (cumsum) and range
        xx=xo+[0;cumsum(dxa)];
        yy=yo+[0;cumsum(dya)];
        zz=zo+[0;0*cumsum(dza)];
        rr=sqrt(xx.^2+yy.^2+zz.^2);
        %
        %off-axis
        oa=acos(sum([xx(2:end),yy(2:end),zz(2:end)].*[dxa,dya,dza],2)./(dro*rr(2:end)));
        %
        % accumulate range off-axis histograms
        hy=hist3([rr(2:end),oa],'edges',{hx, ho});
        Ho=Ho+hy;
    end
    toc
    save('Scr9_7dat','Ho','hx','ho','nsim','nrep');
end
%
Ho=Ho/nrep; %get average
%
%partial CDF
CHo=cumsum(Ho,2);

%plot surfaces
hxx=hx(1:end-1)+dx/2;
hox=ho(1:end-1)+do/2/180*pi;

figure(1)
imagesc(hox*180/pi,hxx,Ho)
xlabel('Off-axis angle [ ^o]')
ylabel('Range [m]')
colorbar
colormap(1-gray(7*2)), caxis(0.07*[0 1])

if 0
figure(2)
imagesc(hox*180/pi,hxx,CHo)
xlabel('Off-axis angle [ ^o]')
ylabel('Range [m]')
colorbar
colormap(1-gray(8*2)), caxis(8*[0 1])
end

% Estimation of the detection function
% nominal figure of merit
FOM=116;
%
% beam pattern parameters
C1=47;
C2=0.218*17.8;
DL_max=C1*C2^2/(1+C2+C2^2);

%Gaussian SL weighting
sig=5;
Wx=@(x,xo) 1/(sqrt(2*pi)*sig)*exp(-1/2*((x-xo)/sig).^2);

%Transmission loss
rTL=max(1,hx(:));
TL=20*log10(rTL)+9.5*rTL/1000;

% click count as function of range
wcl=CHo(:,end);

% needed for integration
dx=0.1;
x=0:dx:(FOM+DL_max+4*sig); %FOM+DL_max+4*sig is assumed to be infinity
%
%warning verbose on % to check proper command
warning off MATLAB:divideByZero %switch off warning in next line
th=asin(1/(2*C2)*(x./(x-C1)).*real(-sqrt(1-4*((x-C1)./x))-1));
warning on all % reactivate warning
%
th(1)=0; % correct for NaN
th(x>=DL_max)=pi; % for x>=DL_max
%
% select integration support
u2=x(x<DL_max);
u3=x(x>=DL_max);
th2=th(x<DL_max);

g2=0*TL;
g3=0*TL;
for ii=1:length(TL)
    W2=Wx(u2+TL(ii),FOM);
    W3=Wx(u3+TL(ii),FOM);
    wth=interp1(ho,CHo(ii,:),th2);
    g2(ii)=sum(W2.*wth)*dx;
    g3(ii)=sum(W3)*dx;
end
%correct for click PDF
iwcl=wcl>0;
g2(iwcl)=g2(iwcl)./wcl(iwcl);
g2(1)=g2(2);

gr=g2+g3;
%
figure(3)
plot(hx,g3,'k')
line(hx,g2,'color','k','linestyle','--')
line(hx,gr,'color','k','linewidth',2)
ylim([0 1.1])
%line([0 7000],[0 1],'color','k','linestyle',':'), ylim([0 0.3])
xlabel('Range [m]')
ylabel('Probability')
return
